//**************************************************************************
//
// Copyright (c) 1997.
//      Richard D. Irwin, Inc.
//
// This software may not be distributed further without permission from
// Richard D. Irwin, Inc.
//
// This software is distributed WITHOUT ANY WARRANTY. No claims are made
// as to its functionality or purpose.
//
// Author: Devon Lockwood
// Date: 1/30/97
// $Revision: 1.1 $
// $Name: $
//
//**************************************************************************
/* EzWindows Library Source File
   FILE:        rect.cc
   AUTHORS:     James P. Cohoon and Jack W. Davidson
   EDITED BY:   Devon Lockwood
   Time-stamp:  <97/01/17 18:42:55 dcl3a>

   Description
   ===========
   The RectangleShape class provides the ability to draw
   rectangles in an EzWindow.
*/
#include <assert.h>
#include "rect.h"


RectangleShape::RectangleShape(SimpleWindow   &iWindow,
                               const Position &iCenter, 
                               const color    &iColor, 
                               float           iLength,
                               float           iWidth) 
                             : Shape (iWindow, iCenter, iColor),
                               Length(iLength), 
                               Width (iWidth) {

}


RectangleShape::RectangleShape(SimpleWindow   &iWindow,
                               float           iXPosition, 
                               float           iYPosition,
                               const color    &iColor, 
                               float           iLength, 
                               float           iWidth) 
                             : Shape(iWindow, 
                                     Position(iXPosition, 
                                              iYPosition), 
                                              iColor),
                               Length(iLength), 
                               Width (iWidth) {

}


float RectangleShape::GetLength() const {
	
   return Length;
}


float RectangleShape::GetWidth() const {
	
   return Width;
}


void RectangleShape::Draw() {
   const Position RectCenter(GetPosition());

   Position UpperLeft (RectCenter + Position(-.5 * Length, -.5 * Width));
   Position LowerRight(RectCenter + Position(.5  * Length,  .5 * Width));
	
   GetWindow().RenderRectangle(UpperLeft, LowerRight, GetColor());
}


void RectangleShape::SetSize(float iLength, float iWidth) {
	
   Length = iLength;
   Width  = iWidth;
}




