//**************************************************************************//
//                                                                          //
// Copyright (c) 1997.                                                      //
//      Richard D. Irwin, Inc.                                              //
//                                                                          //
// This software may not be distributed further without permission from     //
// Richard D. Irwin, Inc.                                                   //
//                                                                          //
// This software is distributed WITHOUT ANY WARRANTY. No claims are made    //
// as to its functionality or purpose.                                      //
//                                                                          //
// Authors: James P. Cohoon and Jack W. Davidson                            //
// Date: 7/15/96                                                            //
// Version: 1.0b                                                            //
//                                                                          //
//**************************************************************************//

#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include "ezwin.h"
#include "utility.h"

// global window
SimpleWindow W("Stock Chart", 12, 9);

// ApiMain(): manage display of a stock chart
int ApiMain() {
	// extract filename and setup stream
	string StockFile = GetFileName();
	ifstream fin(StockFile.c_str());
	if (! fin) {
		cerr << "Cannot open: " << StockFile << endl;
		exit(1);
	}

	// setup
	W.Open();
	float StockHigh = 0;
	int WeekNbr = 0;

	// iteratively process stock-price intervals
	int WeeklyHigh;
	int WeeklyLow;
	while (fin >> WeeklyLow >> WeeklyHigh) {
		// check for valid data
		if (! Valid(WeeklyLow,WeeklyHigh)) {
			cerr << StockFile << ": Bad data for week "
			<< WeekNbr << endl;
			exit(1);
		}

		// have another week of valid data
		++WeekNbr;

		// chart it
		ChartWeek(WeekNbr, WeeklyLow, WeeklyHigh);

		// update the overall high if necessary
		StockHigh = max(StockHigh, WeeklyHigh);
	}

	// display axes
	DrawXAxis(StockFile, StockHigh);
	DrawYAxis(WeekNbr);

	// all done
	return 0;
}

