//**************************************************************************//
//                                                                          //
// Copyright (c) 1997.                                                      //
//      Richard D. Irwin, Inc.                                              //
//                                                                          //
// This software may not be distributed further without permission from     //
// Richard D. Irwin, Inc.                                                   //
//                                                                          //
// This software is distributed WITHOUT ANY WARRANTY. No claims are made    //
// as to its functionality or purpose.                                      //
//                                                                          //
// Authors: James P. Cohoon and Jack W. Davidson                            //
// Date: 7/15/96                                                            //
// Version: 1.0b                                                            //
//                                                                          //
//**************************************************************************//

// Program to train a parts inspector
#include <iostream.h>
#include <assert.h>
#include "random.h"
#include "rect.h"

// Length of display window
const float DisplayLength = 14.0;
const float DisplayHeight = 6.0;

// Vertical position of squares
const float YPosition = DisplayHeight / 2.0;

// Length of the session (60 seconds)
const long TestTime = 60 * 1000L;

// Randomly pick one of the six colors
color GenerateRandomColor() {
	switch (Uniform(0, 5)) {
		case 0: return Red;
		case 1: return Blue;
		case 2: return Green;
		case 3: return Yellow;
		case 4: return Cyan;
		case 5: return Magenta;
	}
}

// Generate the test parts by randomly
// setting the color of the three parts
void GetPartsImages(RectangleShape &P1,
 RectangleShape &P2, RectangleShape &P3) {

	P1.SetColor(GenerateRandomColor());
	P2.SetColor(GenerateRandomColor());
	P3.SetColor(GenerateRandomColor());

	return;
}

// Display the shapes
void DisplayPartsImages(const RectangleShape &P1,
 const RectangleShape &P2, const RectangleShape &P3) {

	P1.Draw();
	P2.Draw();
	P3.Draw();

	return;
}

// Print the results from the training session
void PrintSessionResults(long Time, int Attempts,
 int Correct, int Wrong) {

	cout << "\n\nFor a training period of "
	 << Time / 1000L << " seconds" << endl;
	cout << "Groups viewed: " << Attempts << endl;
	cout << "Correct responses: " << Correct << endl;
	cout << "Incorrect responses: " << Wrong << endl;
	cout << "Percent correct: "
	 << Correct / (float) Attempts * 100 << endl;

	return;
}

// Determine if the appropriate response was given
int CheckResponse(char Response,
 const RectangleShape &P1, const RectangleShape &P2,
 const RectangleShape &P3) {

	if (P1.GetColor() == P2.GetColor()
	 || P1.GetColor() == P3.GetColor()
	 || P2.GetColor() == P3.GetColor())
		return Response == 'a';
	else
		return Response == 'r';
}

int ApiMain() {

	Randomize();
	Console.Initialize();

	// Define objects for scoring the trainee
	int Attempts = 0;   // Number of tests done
	int CorrectResponses = 0;
	int IncorrectResponses = 0;

	// Begin the training session

	// Record starting time
	const long StartTime = GetMilliseconds();
	long ElapsedTime;
	do {
		Camera.GetImage(Image &v);
		Display.DisplayImage(Image &v);
		Response = Console.GetResponse();

		++Attempts;
		if (CheckResponse(Response, Image &v))
			++CorrectResponses;
		else
			++IncorrectResponses;
		ElapsedTime = GetMilliseconds();
	} while ((ElapsedTime - StartTime) < TestTime);

	Console.PrintSessionResults(TestTime, Attempts,
	 CorrectResponses, IncorrectResponses);

	return 0;
}





