#include <stdlib.h>
#include "poker.h"

// Hand(): create empty poker hand
Hand::Hand() {
	return;
}

// GetCard(): return ith card in hand
Card Hand::GetCard(int i) const {
	if ((i >= 1) && (i <= 5))
		return Cards[i-1];
	else {
		cerr << "Bad poker hand reference" << endl;
		exit(1);
	}
}

// SetCard(): set ith card in hand
void Hand::SetCard(int i, const Card &c) {
	if ((i >= 1) && (i <= 5)) {
		Cards[i-1] = c;
	}
	else {
		cerr << "Bad poker hand reference" << endl;
		exit(1);
	}
}


// Quality(): returns rank of poker hand
Rank Hand::Quality() const {
	if (HasStraightFlush())
		return StraightFlush;
	else if (HasFourOfAKind())
		return FourOfAKind;
	else if (HasFullHouse())
		return FullHouse;
	else if (HasFlush())
		return Flush;
	else if (HasStraight())
		return Straight;
	else if (HasThreeOfAKind())
		return ThreeOfAKind;
	else if (HasTwoPair())
		return TwoPair;
	else if (HasOnePair())
		return OnePair;
	else
		return Nothing;
}


// Has functions
bool Hand::HasOnePair() const {
	Hand h(*this);
	h.Sort();
	CardFace f1 = h.GetCard(1).GetFace();
	CardFace f2 = h.GetCard(2).GetFace();
	CardFace f3 = h.GetCard(3).GetFace();
	CardFace f4 = h.GetCard(4).GetFace();
	CardFace f5 = h.GetCard(5).GetFace();
	return (f1 == f2) || (f2 == f3) || (f3 == f4)
	|| (f4 == f5);
}

bool Hand::HasThreeOfAKind() const {
	Hand h(*this);
	h.Sort();
	CardFace f1 = GetCard(1).GetFace();
	CardFace f2 = GetCard(2).GetFace();
	CardFace f3 = GetCard(3).GetFace();
	CardFace f4 = GetCard(4).GetFace();
	CardFace f5 = GetCard(5).GetFace();
	return (f1 == f3) || (f2 == f4) || (f3 == f5);
}

bool Hand::HasFlush() const {
	CardSuit s1 = GetCard(1).GetSuit();
	CardSuit s2 = GetCard(2).GetSuit();
	CardSuit s3 = GetCard(3).GetSuit();
	CardSuit s4 = GetCard(4).GetSuit();
	CardSuit s5 = GetCard(5).GetSuit();
	return (s1 == s2) && (s2 == s3) && (s3 == s4)
	&& (s4 == s5);
}

bool Hand::HasStraightFlush() const {
	return HasStraight() && HasFlush();
}


