//**************************************************************************//
//                                                                          //
// Copyright (c) 1997.                                                      //
//      Richard D. Irwin, Inc.                                              //
//                                                                          //
// This software may not be distributed further without permission from     //
// Richard D. Irwin, Inc.                                                   //
//                                                                          //
// This software is distributed WITHOUT ANY WARRANTY. No claims are made    //
// as to its functionality or purpose.                                      //
//                                                                          //
// Authors: James P. Cohoon and Jack W. Davidson                            //
// Date: 7/15/96                                                            //
// Version: 1.0b                                                            //
//                                                                          //
//**************************************************************************//

#ifndef BIGNUM_H
#define BIGNUM_H

#ifdef USING_CC
#include "bool.h"  
#endif

#include <iostream.h>

class BigNum {
	public:
		// constructors
		BigNum();
		BigNum(int n);
		BigNum(const BigNum &n);
		BigNum(const char s[]);
		// destructor
		~BigNum();
		// assignment operator
		BigNum& operator=(const BigNum &n);
		// facilitators
		BigNum Add(const BigNum &n) const;
		BigNum Multiply(const BigNum &n) const;
		BigNum Subtract(const BigNum &n) const;
		BigNum Divide(const BigNum &n) const;
		void Insert(ostream &sout) const;
		void Extract(ostream &sout) const;
		int Compare(const BigNum &n) const;
	protected:
		// inspectors
		int GetDigit(int i) const;
		int GetNumberDigits() const;
		// mutators
		void SetDigit(int i, int d);
		void SetNumberDigits(int n);
		void GetMemory(int n);
		void ResizeMemory(int n);
		void RemoveLeadingZeros();
		// facilitators
		BigNum LeftDigitShift(int i) const;
		BigNum Scale(int n) const;
	private:
		// data members
		int *Digits;
		int NumberDigits;
};

// auxiliary operators
BigNum operator+(const BigNum &m, const BigNum &n);
BigNum operator-(const BigNum &m, const BigNum &n);
BigNum operator*(const BigNum &m, const BigNum &n);
BigNum operator/(const BigNum &m, const BigNum &n);
bool operator<(const BigNum &m, const BigNum &n);
bool operator>(const BigNum &m, const BigNum &n);
bool operator==(const BigNum &m, const BigNum &n);
bool operator!=(const BigNum &m, const BigNum &n);
bool operator>=(const BigNum &m, const BigNum &n);
bool operator<=(const BigNum &m, const BigNum &n);
ostream& operator<<(ostream &sout, const BigNum &n);
istream& operator>>(istream &sin,BigNum &n);

#endif


