//**************************************************************************//
//                                                                          //
// Copyright (c) 1997.                                                      //
//      Richard D. Irwin, Inc.                                              //
//                                                                          //
// This software may not be distributed further without permission from     //
// Richard D. Irwin, Inc.                                                   //
//                                                                          //
// This software is distributed WITHOUT ANY WARRANTY. No claims are made    //
// as to its functionality or purpose.                                      //
//                                                                          //
// Authors: James P. Cohoon and Jack W. Davidson                            //
// Date: 7/15/96                                                            //
// Version: 1.0b                                                            //
//                                                                          //
//**************************************************************************//

#ifndef BUG_H
#define BUG_H
#include "bitmap.h"
#include "random.h"
//
// class for a simple bug
//
enum Direction {Up, Down, Left, Right };

class Bug {
	public:
		Bug(SimpleWindow &w, int HitsNeeded = 3,
		 int DirectionChangeProbability = 50);
		int IsHit(const Position &MousePosition);
		int IsDying();
		void Create();
		void Kill();
		virtual void Move() = 0;
	protected:
		// Bug inspectors
		SimpleWindow &GetWindow() const;
		Position GetPosition() const;
		Direction GetDirection() const;
		float GetHorizMovement() const;
		float GetVertMovement() const;
		int GetDirectionChangeProbability() const;
		BitMap &GetBmp(const Direction &d);
      const BitMap &GetBmp(const Direction &d) const;
		// Bug mutators
		void SetWindow(SimpleWindow &w);
		void SetDirection(const Direction &d);
		void SetHorizMovement(float h);
		void SetVertMovement(float v);
		void Draw();
		void Erase();
		void SetPosition(const Position &p);
		void ChangeDirection();
		Position NewPosition() const;
		int AtRightEdge() const;
		int AtLeftEdge() const;
		int AtBottomEdge() const;
		int AtTopEdge() const;
		Random GeneratePercentage;
	private:
		// Data members
		SimpleWindow &Window;
		BitMap Bmp[4];
		float HorizMovement;
		float VertMovement;
		int HitsRequired;
		int HitsTaken;
		int DirectionChangeProbability;
		Direction CurrentDirection;
		Position CurrentPosition;
};

class SlowBug : public Bug {
	public:
		SlowBug(SimpleWindow &w, int HitsNeeded = 4,
		 int DirectionChange = 10);
		void Move();
};

class FastBug : public Bug {
	public:
		FastBug(SimpleWindow &w, int HitsNeeded = 3,
		 int DirectionChangeProbability = 20);
		void Move();
};

#endif